﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Diagnostics;
using System.Runtime.InteropServices;
using EA;
using RestSharp;

namespace CollabPrototype
{
    public class Main: EAAddinBase
    {
        // define menu constants
        new const string menuHeader = "-&3D UML Collab";
        const string menuHello = "&Connect";
        const string menuGoodbye = "&Disconnect";
        const string menuAbout = "&About";


        ///
        /// Called Before EA starts to check Add-In Exists
        /// Nothing is done here.
        /// This operation needs to exists for the addin to work
        ///
        /// <param name="Repository" />the EA repository
        /// a string
        public override String EA_Connect(EA.Repository Repository)
        {

            Globals.InstanceGUID = Repository.InstanceGUID;
            Globals.Repository = Repository;
            Globals.ConnectionString = Repository.ConnectionString;

            return "a string";
        }

        ///
        /// Called when user Clicks Add-Ins Menu item from within EA.
        /// Populates the Menu with our desired selections.
        /// Location can be "TreeView" "MainMenu" or "Diagram".
        ///
        /// <param name="Repository" />the repository
        /// <param name="Location" />the location of the menu
        /// <param name="MenuName" />the name of the menu
        ///
        public override object EA_GetMenuItems(EA.Repository Repository, string Location, string MenuName)
        {

            switch (MenuName)
            {
                // defines the top level menu option
                case "":
                    return menuHeader;
                // defines the submenu options
                case menuHeader:
                    string[] subMenus = { menuHello, menuGoodbye, menuAbout };
                    return subMenus;
            }

            return "";
        }

        ///
        /// Called once Menu has been opened to see what menu items should active.
        ///
        /// <param name="Repository" />the repository
        /// <param name="Location" />the location of the menu
        /// <param name="MenuName" />the name of the menu
        /// <param name="ItemName" />the name of the menu item
        /// <param name="IsEnabled" />boolean indicating whethe the menu item is enabled
        /// <param name="IsChecked" />boolean indicating whether the menu is checked
        public override void EA_GetMenuState(EA.Repository Repository, string Location, string MenuName, string ItemName, ref bool IsEnabled, ref bool IsChecked)
        {
            if (IsProjectOpen(Repository))
            {
                switch (ItemName)
                {
                    // define the state of the hello menu option
                    case menuHello:
                        IsEnabled = !Globals.isUserConnected;
                        break;
                    // define the state of the goodbye menu option
                    case menuGoodbye:
                        IsEnabled = Globals.isUserConnected;
                        break;
                    // define the state of the about menu option
                    case menuAbout:
                        IsEnabled = true;
                        break;
                    // there shouldn't be any other, but just in case disable it.
                    default:
                        IsEnabled = false;
                        break;
                }
            }
            else
            {
                // If no open project, disable all menu options
                IsEnabled = false;
            }
        }

        ///
        /// Called when user makes a selection in the menu.
        /// This is your main exit point to the rest of your Add-in
        ///
        /// <param name="Repository" />the repository
        /// <param name="Location" />the location of the menu
        /// <param name="MenuName" />the name of the menu
        /// <param name="ItemName" />the name of the selected menu item
        public override void EA_MenuClick(EA.Repository Repository, string Location, string MenuName, string ItemName)
        {
            switch (ItemName)
            {
                // user has clicked the menuHello menu option
                case menuHello:
                    Globals.InstanceGUID = Repository.InstanceGUID;
                    Globals.Repository = Repository;
                    this.sayHello();
                    break;
                // user has clicked the menuGoodbye menu option
                case menuGoodbye:
                    this.sayGoodbye();
                    break;
                // user has clicked the menuGoodbye menu option
                case menuAbout:
                    this.showAboutDialog();
                    break;
            }
        }

        public override bool EA_OnPostNewElement(EA.Repository Repository, EA.EventProperties Info)
        {   
            //string IDList;
            //string currentUser = Repository.GetCurrentLoginUser(true);
            long count = Info.Count;

            EventProperty ep = Info.Get("ElementID");
            string eName = ep.Name;
            string eID2 = ep.Value;
            int eID = Convert.ToInt32(eID2);
            Element element = Repository.GetElementByID(eID);
            
            if (element.Type == "Class")
            {
                string className = element.Name;
                string guid = element.ElementGUID;
                string[] attributes = new string[2] {"attr1: String","attr2: Integer"};
                string[] methods = new string[2] { "methodA()", "methodB()" };


                var client = new RestClient();
                client.BaseUrl = new Uri("http://uml3d.herokuapp.com");

                var request = new RestRequest("api/diagram", RestSharp.Method.POST);
                
                request.RequestFormat = DataFormat.Json;
                request.AddBody(new { DiagramGUID = guid, Name = className, Methods = methods, Attributes = attributes });

                /*
                var request = new RestRequest("/", RestSharp.Method.GET);
                var request2 = new RestRequest("api/diagram", RestSharp.Method.POST);
                string classDiagram = "{'Name': '" + className + "', 'Methods': [ 'getXY()']}";
                request2.RequestFormat = DataFormat.Json;
                request2.AddBody(classDiagram);
                var request3 = new RestRequest("api/item/{id}", RestSharp.Method.DELETE);
                request3.AddParameter("id", 555, ParameterType.UrlSegment);
                */

                IRestResponse response = client.Execute(request);


                MessageBox.Show("New Class added: " + className);


            }

            return base.EA_OnPostNewElement(Repository, Info);
        }




        public override void EA_OnNotifyContextItemModified(Repository Repository, string GUID, ObjectType ot)
        {

            if (ot == ObjectType.otElement)
            {
                Element element = Repository.GetElementByGuid(GUID);
                if (element.Type == "Class")
                {
                    var client = new RestClient("http://uml3d.herokuapp.com");
                    var request = new RestRequest("api/diagram/{guid}", RestSharp.Method.PATCH);
                    request.AddParameter("guid", GUID, ParameterType.UrlSegment);
                    request.RequestFormat = DataFormat.Json;
                    Collection methods = element.Methods;
                    for (short i = 0; i < methods.Count; i++)
			        {
                        Object method = methods.GetAt(i);
			        }
                    
                    Collection attributes = element.Attributes;
                    request.AddBody(new { guid = GUID, name = element.Name });
                    IRestResponse response = client.Execute(request);
                }

            }
            base.EA_OnNotifyContextItemModified(Repository, GUID, ot);
        }

        public override bool EA_OnPostNewMethod(Repository Repository, EventProperties Info)
        {

            return base.EA_OnPostNewMethod(Repository, Info);
        }
        


        ///
        /// Say Hello to the world
        ///
        private void sayHello()
        {
            FormLogin loginForm = new FormLogin();
            loginForm.ShowDialog();

            Globals.isUserConnected = true;
        }

        ///
        /// Say Goodbye to the world
        ///
        private void sayGoodbye()
        {
            var client = new RestClient("http://uml3d.herokuapp.com");
            var request = new RestRequest("auth/logout", RestSharp.Method.POST);
            request.RequestFormat = DataFormat.Json;
            request.AddBody(new { id = "EA-"+Globals.InstanceGUID});
            IRestResponse response = client.Execute(request);
            MessageBox.Show("You are offline");
            Globals.isUserConnected = false;
        }

        private void showAboutDialog()
        {
            FormAbout aboutForm = new FormAbout();
            aboutForm.ShowDialog();

        }
    }
}
